@@ -723,9 +723,9 @@ getChromInfoFromBiomart <- function(biomart="ENSEMBL_MART_ENSEMBL",
                                                        release=ensembl_release,
                                                        kingdom=kingdom)
     if(is.na(taxonomyId)){
-        taxonomyId <- GenomeInfoDb:::.taxonomyId(organism)
+        taxonomyId <- GenomeInfoDb:::lookup_tax_id_by_organism(organism)
     }else{
-        GenomeInfoDb:::.checkForAValidTaxonomyId(taxonomyId)
+        GenomeInfoDb:::check_tax_id(taxonomyId)
     }
 
     if (!isSingleStringOrNA(miRBaseBuild))
